@@ -1,4 +1,4 @@
-{-# LANGUAGE BangPatterns, CPP, OverloadedStrings #-}
+{-# LANGUAGE BangPatterns, CPP, OverloadedStrings, ScopedTypeVariables #-}
 
 -- Module:      Data.Csv.Encoding
 -- Copyright:   (c) 2011 MailRank, Inc.
@@ -15,8 +15,10 @@ module Data.Csv.Encoding
       HasHeader(..)
     , decode
     , decodeByName
+    , Quoting(..)
     , encode
     , encodeByName
+    , encodeDefaultOrderedByName
 
     -- ** Encoding and decoding options
     , DecodeOptions(..)
@@ -27,6 +29,12 @@ module Data.Csv.Encoding
     , defaultEncodeOptions
     , encodeWith
     , encodeByNameWith
+    , encodeDefaultOrderedByNameWith
+
+    -- ** Encoding and decoding single records
+    , encodeRecord
+    , encodeNamedRecord
+    , recordSep
     ) where
 
 import Blaze.ByteString.Builder (Builder, fromByteString, fromWord8,
@@ -46,6 +54,7 @@ import Data.Word (Word8)
 import Prelude hiding (unlines)
 
 import Data.Csv.Compat.Monoid ((<>))
+import qualified Data.Csv.Conversion as Conversion
 import Data.Csv.Conversion (FromNamedRecord, FromRecord, ToNamedRecord,
                             ToRecord, parseNamedRecord, parseRecord, runParser,
                             toNamedRecord, toRecord)
@@ -97,6 +106,13 @@ encodeByName :: ToNamedRecord a => Header -> [a] -> L.ByteString
 encodeByName = encodeByNameWith defaultEncodeOptions
 {-# INLINE encodeByName #-}
 
+-- | Like 'encodeByName', but header and field order is dictated by
+-- the 'Conversion.header' method.
+encodeDefaultOrderedByName :: (Conversion.DefaultOrdered a, ToNamedRecord a) =>
+                              [a] -> L.ByteString
+encodeDefaultOrderedByName = encodeDefaultOrderedByNameWith defaultEncodeOptions
+{-# INLINE encodeDefaultOrderedByName #-}
+
 ------------------------------------------------------------------------
 -- ** Encoding and decoding options
 
@@ -139,6 +155,13 @@ decodeByNameWith :: FromNamedRecord a
                  -> Either String (Header, Vector a)
 decodeByNameWith !opts = decodeWithP (csvWithHeader opts)
 
+-- | Should quoting be applied to fields, and at which level
+data Quoting
+    = QuoteNone        -- ^ No quotes
+    | QuoteMinimal     -- ^ Quotes according to RFC 4180
+    | QuoteAll         -- ^ Always quote
+    deriving (Eq, Show)
+
 -- | Options that controls how data is encoded. These options can be
 -- used to e.g. encode data in a tab-separated format instead of in a
 -- comma-separated format.
@@ -165,6 +188,9 @@ data EncodeOptions = EncodeOptions
       -- | Include a header row when encoding @ToNamedRecord@
       -- instances.
     , encIncludeHeader :: !Bool
+
+      -- | What kind of quoting should be applied to text fields.
+    , encQuoting :: !Quoting
     } deriving (Eq, Show)
 
 -- | Encoding options for CSV files.
@@ -173,6 +199,7 @@ defaultEncodeOptions = EncodeOptions
     { encDelimiter     = 44  -- comma
     , encUseCrLf       = True
     , encIncludeHeader = True
+    , encQuoting       = QuoteMinimal
     }
 
 -- | Like 'encode', but lets you customize how the CSV data is
@@ -182,7 +209,8 @@ encodeWith opts
     | validDelim (encDelimiter opts) =
         toLazyByteString
         . unlines (recordSep (encUseCrLf opts))
-        . map (encodeRecord (encDelimiter opts) . toRecord)
+        . map (encodeRecord (encQuoting opts) (encDelimiter opts)
+              . toRecord)
     | otherwise = encodeOptionsError
 {-# INLINE encodeWith #-}
 
@@ -205,16 +233,26 @@ encodeOptionsError = error $ "Data.Csv: " ++
         "\") or one of the record separator characters (i.e. \\n or " ++
         "\\r)"
 
-encodeRecord :: Word8 -> Record -> Builder
-encodeRecord delim = mconcat . intersperse (fromWord8 delim)
-                     . map fromByteString . map (escape delim) . V.toList
+-- | Encode a single record, without the trailing record separator
+-- (i.e. newline).
+encodeRecord :: Quoting -> Word8 -> Record -> Builder
+encodeRecord qtng delim = mconcat . intersperse (fromWord8 delim)
+                     . map fromByteString . map (escape qtng delim) . V.toList
 {-# INLINE encodeRecord #-}
 
+-- | Encode a single named record, without the trailing record
+-- separator (i.e. newline), using the given field order.
+encodeNamedRecord :: Header -> Quoting -> Word8 -> NamedRecord -> Builder
+encodeNamedRecord hdr qtng delim =
+    encodeRecord qtng delim . namedRecordToRecord hdr
+
 -- TODO: Optimize
-escape :: Word8 -> B.ByteString -> B.ByteString
-escape !delim !s
-    | B.any (\ b -> b == dquote || b == delim || b == nl || b == cr || b == sp)
-        s = toByteString $
+escape :: Quoting -> Word8 -> B.ByteString -> B.ByteString
+escape !qtng !delim !s
+    | (qtng == QuoteMinimal &&
+        B.any (\ b -> b == dquote || b == delim || b == nl || b == cr || b == sp) s
+      ) || qtng == QuoteAll
+         = toByteString $
             fromWord8 dquote
             <> B.foldl
                 (\ acc b -> acc <> if b == dquote
@@ -240,14 +278,33 @@ encodeByNameWith opts hdr v
     | otherwise = encodeOptionsError
   where
     rows False = records
-    rows True  = encodeRecord (encDelimiter opts) hdr <>
+    rows True  = encodeRecord (encQuoting opts) (encDelimiter opts) hdr <>
                  recordSep (encUseCrLf opts) <> records
     records = unlines (recordSep (encUseCrLf opts))
-              . map (encodeRecord (encDelimiter opts)
-                     . namedRecordToRecord hdr . toNamedRecord)
+              . map (encodeNamedRecord hdr (encQuoting opts) (encDelimiter opts)
+                     . toNamedRecord)
               $ v
 {-# INLINE encodeByNameWith #-}
 
+-- | Like 'encodeDefaultOrderedByNameWith', but lets you customize how
+-- the CSV data is encoded.
+encodeDefaultOrderedByNameWith ::
+    forall a. (Conversion.DefaultOrdered a, ToNamedRecord a) =>
+    EncodeOptions -> [a] -> L.ByteString
+encodeDefaultOrderedByNameWith opts v
+    | validDelim (encDelimiter opts) =
+        toLazyByteString (rows (encIncludeHeader opts))
+    | otherwise = encodeOptionsError
+  where
+    hdr = (Conversion.headerOrder (undefined :: a))
+    rows False = records
+    rows True  = encodeRecord (encQuoting opts) (encDelimiter opts) hdr <>
+                 recordSep (encUseCrLf opts) <> records
+    records = unlines (recordSep (encUseCrLf opts))
+              . map (encodeNamedRecord hdr (encQuoting opts) (encDelimiter opts)
+                     . toNamedRecord)
+              $ v
+{-# INLINE encodeDefaultOrderedByNameWith #-}
 
 namedRecordToRecord :: Header -> NamedRecord -> Record
 namedRecordToRecord hdr nr = V.map find hdr
